#ifdef GL_ES
precision lowp float;
#endif

varying vec4 v_fragmentColor;
varying vec2 v_texCoord;
varying vec2 v_radial_uv;

uniform float u_bloomIntensity;
uniform sampler2D u_bloomTexture;
uniform float u_sampleDist;
uniform float u_sampleStrength;
uniform vec2 u_center;

void main()
{
    vec4 texColor = texture2D(CC_Texture0, v_texCoord);
    vec4 bloomColor = texture2D(u_bloomTexture, v_texCoord);
    
    float samples[10];
    samples[0] = -0.08;
    samples[1] = -0.05;
    samples[2] = -0.03;
    samples[3] = -0.02;
    samples[4] = -0.01;
    samples[5] =  0.01;
    samples[6] =  0.02;
    samples[7] =  0.03;
    samples[8] =  0.05;
    samples[9] =  0.08;
    
    vec2 dir = u_center - v_radial_uv;
    float dist = sqrt(dir.x*dir.x + dir.y*dir.y);
    dir = dir/dist;
    
    vec4 color = texColor + u_bloomIntensity * bloomColor;
    vec4 sum = color;
    
    vec2 coord1 = v_radial_uv + dir * -0.08 * u_sampleDist;
    vec2 coord2 = v_radial_uv + dir * -0.05 * u_sampleDist;
    vec2 coord3 = v_radial_uv + dir * -0.03 * u_sampleDist;
    vec2 coord4 = v_radial_uv + dir * -0.02 * u_sampleDist;
    vec2 coord5 = v_radial_uv + dir * -0.01 * u_sampleDist;
    vec2 coord6 = v_radial_uv + dir * 0.01 * u_sampleDist;
    vec2 coord7 = v_radial_uv + dir * 0.02 * u_sampleDist;
    vec2 coord8 = v_radial_uv + dir * 0.03 * u_sampleDist;
    vec2 coord9 = v_radial_uv + dir * 0.05 * u_sampleDist;
    vec2 coord10 = v_radial_uv + dir * 0.08 * u_sampleDist;
    
    sum += texture2D(CC_Texture0, coord1) + u_bloomIntensity * texture2D(u_bloomTexture, coord1);
    sum += texture2D(CC_Texture0, coord2) + u_bloomIntensity * texture2D(u_bloomTexture, coord2);
    sum += texture2D(CC_Texture0, coord3) + u_bloomIntensity * texture2D(u_bloomTexture, coord3);
    sum += texture2D(CC_Texture0, coord4) + u_bloomIntensity * texture2D(u_bloomTexture, coord4);
    sum += texture2D(CC_Texture0, coord5) + u_bloomIntensity * texture2D(u_bloomTexture, coord5);
    sum += texture2D(CC_Texture0, coord6) + u_bloomIntensity * texture2D(u_bloomTexture, coord6);
    sum += texture2D(CC_Texture0, coord7) + u_bloomIntensity * texture2D(u_bloomTexture, coord7);
    sum += texture2D(CC_Texture0, coord8) + u_bloomIntensity * texture2D(u_bloomTexture, coord8);
    sum += texture2D(CC_Texture0, coord9) + u_bloomIntensity * texture2D(u_bloomTexture, coord9);
    sum += texture2D(CC_Texture0, coord10) + u_bloomIntensity * texture2D(u_bloomTexture, coord10);
 
    sum *= 1.0/11.0;
    float t = dist * u_sampleStrength;
    t = clamp(t, 0.0, 1.0);
    
    gl_FragColor = mix(color, sum, t);
}