#ifdef GL_ES
precision lowp float;
#endif

varying vec4 v_fragmentColor;
varying vec2 v_texCoord;
varying vec2 v_texCoordRefraction;

uniform float u_brightActive;
uniform sampler2D u_bloomTexture;
uniform sampler2D u_refractionTexture;
uniform vec2 u_shockWaveCenter;
uniform vec4 u_shockWaveParams;
uniform float u_bloomIntensity;
uniform sampler2D u_heatTexture;
uniform float u_heatActive;
uniform float u_heatTime;
uniform vec2 u_texSize;

float luminance(vec2 uv)
{
    return dot(texture2D(CC_Texture0, uv).rgb, vec3(0.2126, 0.7152, 0.0722));
}

float custom_fwidth(vec2 uv)
{
    vec2 pixelStep = vec2(1.0 / u_texSize.x, 1.0 / u_texSize.y);
    
    float current = luminance(uv);
    float dfdx = luminance(uv + pixelStep.x) - current;
    float dfdy = luminance(uv + pixelStep.y) - current;
    
    return abs(dfdx) + abs(dfdy);
}

void main()
{
    // Onda de choque
    float waveTime = u_shockWaveParams.x / u_shockWaveParams.y;
    float freq = u_shockWaveParams.z;
    vec2 center = u_shockWaveCenter;
    float speed = u_shockWaveParams.w;
    
    float dist = length(v_texCoord - center);
    vec2 p = freq * (v_texCoord - center);
    float len = length(p);
    // Si se hace la división por len, en algunos dispositivos se ve un cuadro negro (cuando len == 0.0). Se hace una aproximación.
    // (1.0 - waveTime) es el tiempo
    //vec2 uv = v_texCoord + (p / len) * cos(len * 12.0 - (1.0 - waveTime) * speed) * 0.06 * (1.0 - dist) * waveTime;
    vec2 uv = v_texCoord + (p * 6.0 * (1.0 - len)) * cos(len * 12.0 - (1.0 - waveTime) * speed) * 0.06 * (1.0 - dist) * waveTime;
    
    // Desplazamiento de la textura de refracción.
    vec4 refractionColor = texture2D(u_refractionTexture, v_texCoordRefraction);
    vec2 dispTexOffset = vec2((refractionColor.g - 0.647) * 0.03, (refractionColor.r - 0.647) * 0.05);
    uv = uv + dispTexOffset;
    
    // Acceso al color real y ponerlo más brillante en las crestas de la onda de choque
    float diff = length(v_texCoord - uv);
    float shockWaveBright = diff * diff * 100.0;
    vec4 texColor = texture2D(CC_Texture0, uv);// + vec4(bright, bright, bright, 0.0);
    texColor = texColor + 3.0 * shockWaveBright * texColor + max(0.0, 3.0 * dispTexOffset.x + 5.0 * dispTexOffset.y) * texColor;
    
    // Brillo
    vec4 activeColor = 1.15 * texColor;
    vec4 finalColor = mix(texColor, activeColor, u_brightActive);
    
    // Bloom
    vec4 bloomColor = texture2D(u_bloomTexture, uv);
    
    // Detección de bordes
    float edge = custom_fwidth(uv);
    // Lo comentado sirve para openGL ES 3.0 y para openGL ES 2.0 con la extensión de derivativas activa. El problema es que puede haber dispositivos que no soportan la extensión.
    //float luminance = dot(finalColor.rgb, vec3(0.2126, 0.7152, 0.0722));
    //fwidth(luminance); abs(dFdx(p)) + abs(dFdy(p))
    
    /// Color con efecto de bloom
    vec4 bloomedColor = finalColor + u_bloomIntensity * bloomColor + edge * u_bloomIntensity * bloomColor;
    
    /// Distorsión heathaze
    vec2 heatUV = vec2(uv.x - u_heatTime, uv.y - u_heatTime);
    vec4 heatNoise = texture2D(u_heatTexture, heatUV);
    vec2 dst_offset = heatNoise.xy;
    dst_offset -= vec2(0.5, 0.5);
    dst_offset *= 0.8 * (1.0 - uv.y);
    vec2 dist_tex_coord = uv + dst_offset;
    vec4 heatTexColor = texture2D(CC_Texture0, dist_tex_coord);
    vec4 heatActiveColor = 1.1 * heatTexColor;
    vec4 heatFinalColor = mix(heatTexColor, heatActiveColor, u_brightActive);
    vec4 heatBloomColor = texture2D(u_bloomTexture, dist_tex_coord);
    float heatEdge = custom_fwidth(uv);
    //float heatLuminance = dot(heatFinalColor.rgb, vec3(0.2126, 0.7152, 0.0722));
    //float heatEdge = fwidth(heatLuminance);
    vec4 heatedColor = heatFinalColor + u_bloomIntensity * heatBloomColor + heatEdge * u_bloomIntensity * heatBloomColor;
    
    gl_FragColor = mix(bloomedColor, heatedColor, pow(uv.y, 1.5) * u_heatActive);
    
    // Color sin heatHaze
//    gl_FragColor = bloomedColor;
    
    // Textura de luces
//    gl_FragColor = bloomColor;
    
    // Screen blending
//    gl_FragColor = vec4(1.0 - (1.0 - finalColor.rgb) * (1.0 - bloomColor.rgb), 1.0);
    
    // Overlay blending
//    float grayscale = dot(finalColor.rgb, vec3(0.299, 0.587, 0.114));
//    if(grayscale < 0.5)
//    {
//        gl_FragColor = vec4(2.0 * finalColor.rgb * bloomColor.rgb, 1.0);
//    }
//    else
//    {
//        gl_FragColor = vec4(1.0 - 2.0 * (1.0 - finalColor.rgb) * (1.0 - bloomColor.rgb), 1.0);
//    }
}
