#ifdef GL_ES
precision lowp float;
#endif

varying vec4 v_fragmentColor;
varying vec2 v_texCoord;

uniform float u_brightActive;
uniform sampler2D u_bloomTexture;
uniform vec2 u_shockWaveCenter;
uniform vec4 u_shockWaveParams;
uniform float u_bloomIntensity;
uniform sampler2D u_heatTexture;
uniform float u_heatActive;
uniform vec2 u_texSize;

void main()
{
    // Onda de choque
    float waveTime = u_shockWaveParams.x / u_shockWaveParams.y;
    float freq = u_shockWaveParams.z;
    vec2 center = u_shockWaveCenter;
    float speed = u_shockWaveParams.w;
    
    float dist = length(v_texCoord - center);
    vec2 p = freq * (v_texCoord - center);
    float len = length(p);
    // Si se hace la división por len, en algunos dispositivos se ve un cuadro negro (cuando len == 0.0). Se hace una aproximación.
    // (1.0 - waveTime) es el tiempo
    //vec2 uv = v_texCoord + (p / len) * cos(len * 12.0 - (1.0 - waveTime) * speed) * 0.06 * (1.0 - dist) * waveTime;
    vec2 uv = v_texCoord + (p * 6.0 * (1.0 - len)) * cos(len * 12.0 - (1.0 - waveTime) * speed) * 0.06 * (1.0 - dist) * waveTime;
    
    // Acceso al color real y ponerlo más brillante en las crestas de la onda de choque
    float diff = length(v_texCoord - uv);
    float shockWaveBright = diff * diff * 100.0;
    vec4 texColor = texture2D(CC_Texture0, uv);// + vec4(bright, bright, bright, 0.0);
    texColor = texColor + 3.0 * shockWaveBright * texColor;
    
    // Brillo
    vec4 activeColor = 1.15 * texColor;
    vec4 finalColor = mix(texColor, activeColor, u_brightActive);
    
    // Bloom
    vec4 bloomColor = texture2D(u_bloomTexture, uv);
    
    /// Color con efecto de bloom
    vec4 bloomedColor = finalColor + u_bloomIntensity * bloomColor;
    
    /// Distorsión heathaze
    vec2 heatUV = vec2(uv.x - CC_Time.g * 0.3, uv.y - CC_Time.g * 0.3);
    vec4 heatNoise = texture2D(u_heatTexture, heatUV);
    vec2 dst_offset = heatNoise.xy;
    dst_offset -= vec2(0.5, 0.5);
    dst_offset *= 0.8 * (1.0 - uv.y);
    vec2 dist_tex_coord = uv + dst_offset;
    vec4 heatTexColor = texture2D(CC_Texture0, dist_tex_coord);
    
    gl_FragColor = mix(bloomedColor, heatTexColor, pow(uv.y, 1.5) * u_heatActive);
    
    // Color sin heatHaze
//    gl_FragColor = bloomedColor;
    
    // Textura de luces
//    gl_FragColor = bloomColor;
    
    // Screen blending
//    gl_FragColor = vec4(1.0 - (1.0 - finalColor.rgb) * (1.0 - bloomColor.rgb), 1.0);
    
    // Overlay blending
//    float grayscale = dot(finalColor.rgb, vec3(0.299, 0.587, 0.114));
//    if(grayscale < 0.5)
//    {
//        gl_FragColor = vec4(2.0 * finalColor.rgb * bloomColor.rgb, 1.0);
//    }
//    else
//    {
//        gl_FragColor = vec4(1.0 - 2.0 * (1.0 - finalColor.rgb) * (1.0 - bloomColor.rgb), 1.0);
//    }
}
